/** Always throws. */
declare function fatal(...messages: any[]): never;
/** Throws if condition is not truthy. */
declare function assert(condition: unknown, ...messages: any[]): asserts condition;
/** Alias for `assert` for better differentiation to unit test's assert function. */
declare function assertCondition(condition: unknown, ...messages: any[]): asserts condition;

interface Encoder {
    encode(data: any): Promise<Uint8Array>;
    decode(data: Uint8Array): Promise<any>;
}
declare class NoopEncoder implements Encoder {
    encode(data: any): Promise<Uint8Array>;
    decode(data: Uint8Array): Promise<any>;
}
declare class JsonEncoder implements Encoder {
    encode(data: any): Promise<Uint8Array>;
    decode(data: Uint8Array): Promise<any>;
}
declare class CryptoEncoder implements Encoder {
    key: CryptoKey;
    constructor(key: CryptoKey);
    encode(data: any): Promise<Uint8Array>;
    decode(data: Uint8Array): Promise<any>;
}

type BinInput = Uint8Array | ArrayBuffer | string | number[];
declare function _encodeUtf8Polyfill(str: string): Uint8Array;
declare function _decodeUtf8Polyfill(buf: Uint8Array): string;
declare function stringToUInt8Array(text: string): Uint8Array;
declare function Uint8ArrayToString(bin: Uint8Array): string;
declare function toUint8Array(data: BinInput): Uint8Array;
declare function joinToUint8Array(...args: BinInput[] | BinInput[][]): Uint8Array;
declare function toHex(bin: BinInput): string;
declare function fromHex(hexString: string): Uint8Array;
/** Regular base64 */
declare function toBase64(bin: BinInput, stripPadding?: boolean): string;
declare function toBase64Url(bin: BinInput): string;
declare function fromBase64(s: string): Uint8Array;
declare function fromBase64String(s: string): string;
/** Compare contents of binary arrays */
declare function equalBinary(a: ArrayBuffer | Uint8Array, b: ArrayBuffer | Uint8Array): boolean;
declare function jsonToUint8Array(json: any): Uint8Array | never;
declare function Uint8ArrayToJson<T = any>(data: Uint8Array): T | never;
declare function Uint8ArrayToHexDump(buffer: Uint8Array | ArrayBuffer | String | Array<number>, blockSize?: number): string | false;

declare class BinaryEncoder implements Encoder {
    encode(data: any): Promise<Uint8Array>;
    decode(data: Uint8Array | ArrayBuffer): Promise<any>;
}
/** Encode any object, including Uint8Array data */
declare function encodeJson(data: any): Uint8Array;
/** Decode any object, including Uint8Array data */
declare function decodeJson(data: Uint8Array | ArrayBuffer): any;
/** Incrementally encode binary data */
declare function createBinaryStreamEncoder(initialData?: BinInput): {
    /** Just writes the bytes. Length is not stored! */
    writeBytes: (data: BinInput, expectedLength: number) => void;
    writeBytesVar: (data: BinInput) => void;
    writeUintVar: (data: number) => void;
    writeUint8: (data: number) => void;
    writeUint16: (data: number) => void;
    writeUint32: (data: number) => void;
    writeIntVar: (data: number) => void;
    writeFloat: (data: number) => void;
    writeStringVar: (data: string) => void;
    write: (data: any) => void;
    getUint8Array: () => Uint8Array;
    getLength: () => number;
};
/** Incrementally decode binary data */
declare function createBinaryStreamDecoder(data: BinInput): {
    readBytes: (len: number) => Uint8Array;
    readBytesVar: () => Uint8Array;
    readUintVar: () => number;
    readUint8: () => number;
    readUint16: () => number;
    readUint32: () => number;
    readIntVar: () => number;
    readFloat: () => number;
    readStringVar: () => string;
    read: () => any;
    getOffset: () => number;
};
declare function bitfield(bitfield?: number): {
    setBit: (bit: number, value?: boolean) => number;
    unsetBit: (bit: number) => number;
    toggleBit: (bit: number) => number;
    getBit: (bit: number) => boolean;
    getValue: () => number;
};
declare function blobToArrayBuffer(blob: Blob | File): Promise<ArrayBuffer | undefined>;
declare function blobToUint8Array(blob: Blob | File): Promise<Uint8Array | undefined>;

/** Get random bytes using window.crypto if available. Else use a poor fallback solution. */
declare function randomUint8Array(length?: number): Uint8Array;
declare const CRYPTO_DEFAULT_HASH_ALG = "SHA-256";
declare const CRYPTO_DEFAULT_ALG = "AES-GCM";
declare const CRYPTO_DEFAULT_DERIVE_ALG = "PBKDF2";
declare const CRYPTO_DEFAULT_DERIVE_ITERATIONS = 100000;
declare function digest(message: BinInput, algorithm?: AlgorithmIdentifier): Promise<Uint8Array>;
declare function deriveKeyPbkdf2(secret: BinInput, opt?: {
    iterations?: number;
    salt?: BinInput;
}): Promise<CryptoKey>;
declare function encrypt(data: Uint8Array, key: CryptoKey): Promise<Uint8Array>;
declare function decrypt(data: Uint8Array, key: CryptoKey): Promise<Uint8Array>;

declare function csvStringify(data: any[], opt?: {
    separator?: string;
}): string;
declare function csvParse(raw: string, opt?: {
    separator?: string;
}): any[][];
declare function csvParseToObjects(raw: string, opt?: {
    separator?: string;
}): any[];

type Json = string | number | boolean | null | {
    [property: string]: Json;
} | Json[];
interface ObjectStorage<T = any> {
    setItem(key: string, value: T): void;
    getItem(key: string): T | undefined;
    removeItem(key: string): void;
    clear(): void;
    allKeys(): string[];
}
type NestedArray<T> = T | Array<NestedArray<T>>;

declare function arrayUnique<T>(x: T[]): T[];
declare function arrayMinus<T>(x: T[], y: T[]): T[];
declare function arrayUnion<T>(...a: T[][]): T[];
/** `[1,[2,3]]` becomes `[1,2,3]` */
declare function arrayFlatten<T>(...list: NestedArray<T>[]): T[];
declare function arrayIntersection<T>(x: T[], y: T[]): T[];
declare function arraySymmetricDifference<T>(x: T[], y: T[]): T[];
declare function arrayRemoveElement<T>(arr: T[], el: T): T[];
/** Only have it once in the set */
declare function arraySetElement<T>(arr: T[], el: T): T[];
declare function arrayFilterInPlace<T>(array: T[], fn: (el: T) => boolean): T[];
declare function arrayToggleInPlace<T>(array: T[], el: T): T[];
declare function arrayEmptyInPlace<T>(array: T[]): T[];
declare function arraySorted<T>(arr: Iterable<T> | ArrayLike<T>, cond?: ((a: T, b: T) => number) | undefined): T[];
declare function arraySortedNumbers(arr: number[]): number[];
declare function arrayIsEqual<T>(array1: T[], array2: T[]): boolean;
declare function arrayShuffleInPlace<T>(array: T[]): T[];
declare function arrayShuffle<T>(array: T[]): T[];
/** Randomly shuffle the order of the array's elements. Force to have a different order if array has more than one element. */
declare function arrayShuffleForce<T>(array: T[]): T[];
declare function arrayRandomElement<T>(array: T[]): T;
declare function arrayMax<T>(...array: NestedArray<T>[]): T;
declare function arrayMin<T>(...array: NestedArray<T>[]): T;
declare function arraySum(...array: NestedArray<number>[]): number;
declare function arrayAvg(...array: NestedArray<number>[]): number;
declare function arrayBatches<T>(array: T[], chunckLength: number): T[][];
declare function createArray<T>(size?: number, item?: T | ((index: number) => T)): T[];

type encodeFunction = (source: BinInput, padToLength?: number) => string;
type decodeFunction = (source: string, padToLength?: number) => Uint8Array;
declare function _useBase(alphaOrBase: string | number): {
    encode: encodeFunction;
    decode: decodeFunction;
};
declare function useBase(alphaOrBase: string | number): {
    encode: (source: BinInput, padToLength?: number) => string;
    decode: (source: string, padToLength?: number) => Uint8Array;
};
declare function encodeBase32(bin: BinInput, padding?: number): string;
declare function decodeBase32(s: string, padding?: number): Uint8Array;
declare function encodeBase16(bin: BinInput, padding?: number): string;
declare function decodeBase16(s: string, padding?: number): Uint8Array;
declare function encodeBase58(bin: BinInput, padding?: number): string;
declare function decodeBase58(s: string, padding?: number): Uint8Array;
declare function encodeBase62(bin: BinInput, padding?: number): string;
declare function decodeBase62(s: string, padding?: number): Uint8Array;
declare function estimateSizeForBase(bytes: number, base: number): number;

declare const toCamelCase: (s: string) => string;
declare function toCapitalize(s: string): string;
declare function toCapitalizeWords(s: string): string;
declare function fromCamelCase(str: string, separator?: string): string;

declare function stringToBoolean(value?: string, defaultValue?: boolean): boolean;
declare function stringToInteger(value?: string, defaultValue?: number): number;
declare function stringToFloat(value?: string, defaultValue?: number): number;
declare function valueToBoolean(value?: any, defaultValue?: boolean): boolean;
declare function valueToInteger(value?: any, defaultValue?: number): number;
declare function valueToFloat(value?: any, defaultValue?: number): number;
declare function valueToString(value?: any, defaultValue?: string): string;
declare const toFloat: typeof valueToFloat;
declare const toInt: typeof valueToInteger;
declare const toString: typeof valueToString;
declare const toBool: typeof valueToBoolean;
interface RenderMessagesOptions {
    trace?: boolean;
    pretty?: boolean;
}
declare function formatMessages(messages: any[], opt?: RenderMessagesOptions): any[];
declare function renderMessages(messages: any[], opt?: RenderMessagesOptions): string;
declare function fixBrokenUth8String(brokenString: string): string;

declare const DAY_MS = 86400000;
type DayValue = number;
type DayInput = DayValue | number | string | Date | [number, number?, number?];
declare function dayYear(day: DayValue): DayValue;
declare function dayMonth(day: DayValue): DayValue;
declare function dayDay(day: DayValue): DayValue;
declare function dayToParts(day: DayValue): [number, number, number];
declare function dayToDate(day: DayValue, gmt?: boolean): Date;
declare function dayFromToday(): DayValue;
declare function dayFromAny(value: DayInput, gmt?: boolean): DayValue | undefined;
declare function dayToDateGMT(day: DayValue): Date;
declare function dayFromDate(date: Date, gmt?: boolean): DayValue;
declare function dayFromDateGMT(date: Date): DayValue;
declare function dayToTimestamp(day: DayValue, gmt?: boolean): number;
declare function dayFromTimestamp(ms: number, gmt?: boolean): DayValue;
declare function dayToString(day: DayValue, sep?: string): string;
declare function dayFromParts(year: number, month?: number, day?: number): DayValue | undefined;
declare function dayFromString(value: string): DayValue | undefined;
declare function dayMonthStart(day: DayValue, offset?: number): DayValue;
declare function dayYearStart(day: DayValue, offset?: number): DayValue;
declare function dayOffset(day: DayValue, offset: number): DayValue;
declare function dayDiff(left: DayValue, right: DayValue): number;
/** List of dates for loops */
declare function dayRange(left: DayValue, right?: DayValue): number[];
/** Iterator, see dayRange */
declare function dayIterator(left: DayValue, right?: DayValue): Generator<number, number[], unknown>;

type DayInputLegacy = DayInput | Day;
/**
 * Date represented as positive integer value. Years smaller 0 are not supported.
 * @deprecated
 */
declare class Day {
    days: DayValue;
    get value(): number;
    constructor(days?: DayInputLegacy);
    static fromNumber(n: number): Day;
    static fromString(dateString: string): Day | undefined;
    static fromDate(date: Date, gmt?: boolean): Day;
    static fromDateGMT(date: Date): Day;
    static from(value: DayInputLegacy, gmt?: boolean): Day | undefined;
    toNumber(): number;
    /** Just for future extensions */
    toJson(): number;
    toString(sep?: string): string;
    toDate(gmt?: boolean): Date;
    toDateGMT(): Date;
    get year(): number;
    get month(): number;
    get day(): number;
    dayOffset(offset: number): Day;
    monthStart(): Day;
    yearStart(): Day;
    /** Very stupid approach, only works for days <= 28 */
    monthOffset(offset: number): Day;
    daysUntil(otherDay: DayInputLegacy): number;
    yesterday(): Day;
    tomorrow(): Day;
}
declare function forEachDay(from: DayInputLegacy, to: DayInputLegacy, handler: (date: Day) => Promise<void> | void): Promise<void>;
declare function today(): Day;
/** @deprecated */
declare function day(days?: DayInputLegacy): Day;
declare function dateStringToDays(dateString: string): number;

type DecimalValue = number;
type DecimalInput = number | string | DecimalValue;
declare function decimal(value: DecimalInput, decimalPlaces?: number): DecimalValue;
declare function decimalFromCents(value: DecimalInput, decimalPlaces?: number): DecimalValue;
declare function decimalToCents(value: DecimalInput, decimalPlaces?: number): number;
declare function decimalCentsPart(value: DecimalInput, decimalPlaces?: number): DecimalValue;

declare function deepEqual(a: any, b: any, hash?: WeakSet<object>): boolean;
/** Strip properties with value `undefined` in place */
declare function deepStripUndefinedInPlace(a: any, hash?: WeakSet<object>): any;
declare function deepMerge(target: any, ...sources: any[]): any;

type DifferenceType = 'new' | 'upd' | 'del';
interface Difference {
    type: DifferenceType;
    path: (string | number)[];
    value?: any;
}
interface Options {
    cyclesFix: boolean;
}
/**
 * Describes the changes between to object with a list like:
 *
 * ```
 * [{
 *    type: 'upd',
 *    path: ['a'],
 *    value: 1
 * },
 * {
 *    type: 'del',
 *    path: ['b', 'c']
 * }
 * ...]
 * ```
 */
declare function diffObjects(obj: Record<string, any> | any[], newObj: Record<string, any> | any[], options?: Partial<Options>, _stack?: Record<string, any>[]): Difference[];

declare const escapeHTML: (s: string) => string;
declare const unescapeHTML: (s: string) => string;

type Primitive = null | undefined | string | number | boolean | symbol | bigint;
declare function isObject(obj: unknown): obj is object;
/** Something like number, string, boolean */
declare function isPrimitive(obj: unknown): obj is Primitive;
declare function isArray(obj: unknown): obj is Array<any>;
/** Object that is not an array. But could also be an object defined by a class. */
declare function isRecord(obj: unknown): obj is Record<string, any>;
/** Just data, like constructed via `{...}`. */
declare function isRecordPlain(obj: unknown): obj is Record<string, any>;
declare function isString(obj: unknown): obj is string;
declare function isNumber(obj: unknown): obj is number;
declare function isInteger(obj: unknown): obj is number;
/** Integer with full precision i.e. its value is in the signed 53 bit range.  */
declare function isSafeInteger(obj: unknown): obj is number;
declare function isBoolean(obj: unknown): obj is boolean;
/** @deprecated use `isNull` */
declare function isNullOrUndefined(obj: unknown): obj is null | undefined;
declare function isNull(obj: unknown): obj is null | undefined;
declare function isUint8Array(obj: unknown): obj is Uint8Array;
declare function isNotNull<T>(value: T | null | undefined): value is T;
declare function isValue<T>(value: T | null | undefined | boolean): value is T;

declare function jsonStringifySorted(obj: any, indent?: number | undefined): string;
type EntryProcessor = (key: string, value: any) => any;
/**
 * Similar to JSON.stringify but can handle circular references
 */
declare function jsonStringifySafe(obj: any, replacer?: EntryProcessor | null, spaces?: string | number | null, cycleReplacer?: EntryProcessor): string;
declare const jsonStringify: typeof jsonStringifySafe;

type FilterFunction<T> = (o: T) => boolean;
type MapperFunction<T> = (o: T) => any;
/** Apply multiple filters, optionally transform through mappers, filter null/undefined items. */
declare function listQuery<T>(list: T[], filters: FilterFunction<T>[], mappers?: MapperFunction<T>[]): T | any[];
/** Split up a list by `key` resulting in a Record of `key` and sub-list. */
declare function listGroupBy<T extends Record<string, any>>(list: T[], key: keyof T): Record<string, T[]>;
/** Returns a list of values of a certain `key`. No duplicates. */
declare function listDistinctUnion<T extends Record<string, any>>(list: T[], key: keyof T): any[];
/** Returns a list of values of a certain `key`. */
declare function listOfKey<T extends Record<string, any>>(list: T[], key: keyof T): any[];

declare function getSecureRandom(): number;
declare function getSecureRandomIfPossible(): number;
declare function randomBoolean(bias?: number): boolean;
/** max is not included, min is included */
declare function randomInt(max?: number, min?: number): number;
declare function randomFloat(max?: number, min?: number): number;
declare function between(min: number, value: number, max: number): number;
/** See also arraySum */
declare function sum(array: number[]): number;
/** See also arrayAvg */
declare function avg(array: number[]): number;
/** Fancy prime number check ;) */
declare function isPrimeRX(value: number): boolean;
declare function isPrime(value: number): boolean;
/** Deterministic random */
declare function seededRandom(max?: number, min?: number, seed?: number): number;

/** Like `.map()` for object. Return new key and value or `undefined` to delete. */
declare function objectMap<T = any>(obj: any, fn: (key: string, value: any) => [key: string, value: T] | T | undefined | null): Record<string, T>;
/** Merge right into left object. If dispose is defined, it will be combined. Left can be a complex object i.e. a insantiated class. */
declare function objectMergeDisposable<A extends object, B extends object>(a: A, b: B): A & B;
declare function objectIsEmpty(obj: object): boolean;

declare function parseOrderby(value?: string): {
    field: string;
    orderby: string;
    asc: boolean;
    desc: boolean;
};
declare function composeOrderby(field: string, asc?: boolean): string;
declare function cmp(a: any, b: any, asc?: boolean): -1 | 0 | 1;
declare function sortedOrderby<T>(values: T[], ...orderby: string[]): T[];

/** Ensure a valid file name. Unsupported characters get a `replacement`, default: `_` */
declare function toValidFilename(string: string, replacement?: string): string | never;

declare function escapeRegExp(value: RegExp | string): string;

type RoundingMode = (value: number) => number;
declare const isHalf: (value: number) => boolean;
declare const isEven: (value: number) => boolean;
declare const roundUp: RoundingMode;
declare const roundDown: RoundingMode;
declare const roundHalfUp: RoundingMode;
/**
 * Round a number with half values to nearest odd integer.
 */
declare const roundHalfOdd: RoundingMode;
/**
 * Round a number with half values to nearest integer farthest from zero.
 */
declare const roundHalfAwayFromZero: RoundingMode;
/**
 * Round a number with half values down.
 */
declare const roundHalfDown: RoundingMode;
/**
 * Round a number with half values to nearest even integer.
 * https://wiki.c2.com/?BankersRounding
 */
declare const roundHalfEven: RoundingMode;
/**
 * Round a number with half values to nearest integer closest to zero.
 */
declare const roundHalfTowardsZero: RoundingMode;

interface SortableItem {
    sort_weight: number;
}
declare function startSortWeight(items: SortableItem[]): number;
declare function endSortWeight(items: SortableItem[]): number;
declare function moveSortWeight(newIndex: number, oldIndex: number, items: SortableItem[]): number;
declare function sortedItems<T extends SortableItem>(items: T[]): T[];

/** CRDT compatible sorting with a single operation. Default name is `sort` */
declare function useSorted<S extends Record<string, any>>(config?: {
    getter?: (item: S) => number;
}): {
    start: (items: S[]) => number;
    end: (items: S[]) => number;
    between: (lower?: number, upper?: number) => number;
    move: (newIndex: number, oldIndex: number, sortableItems: S[]) => number;
    items: <T extends S>(items: T[]) => T[];
};

declare function linkifyPlainText(text: string): string;
declare function toHumanReadableUrl(url: string): string;
declare function encodeQuery(data: Record<string, any>): string;
declare function parseQuery(queryString: string): any;

/**
 * Call a create function if key does not yet exist on an object. Returns the found or created object. Example:
 *
 * ```js
 * function createRoom(room, rooms) { return new Room() }
 * ensureKey(rooms, room, createRoom).enter()
 * ```
 */
declare function ensureKey<T>(obj: Record<string, T>, key: string, createFn: (key?: string, obj?: Record<string, T>) => T): T;
/**
 * Call a create function if key does not yet exist on an object. Returns the found or created object. Example:
 *
 * ```js
 * async function fetchItem(id, cache) { ... }
 * let data = await ensureKey(cache, id, fetchItem)
 * ```
 */
declare function ensureKeyAsync<T>(obj: Record<string, T>, key: string, createFn: (key?: string, obj?: Record<string, T>) => Promise<T>): Promise<T>;
declare function size(obj: any): any;
declare function last<T>(array?: T[]): T | undefined;
declare function empty(value: any): boolean;
declare function cloneObject<T>(obj: T): T;
declare function cloneJsonObject<T = Json>(obj: T): T;
/**
 * Cache result of a function. Same arguments have to always return the same result in order to get expected optimization!
 *
 * ```
 * const square = memoize((value) => value * value)`
 * square(2) // == 2
 * ```
 */
declare function memoize<In, Out>(fn: (arg: In) => Out): (arg: In) => Out;
declare function memoizeAsync<In extends any[], Out extends Promise<any>>(fn: (...arg: In) => Out): (...arg: In) => Promise<Out>;
/** Repeat `count` times. Starts with `0` */
declare function forTimes<T = undefined>(count: number, fn: (i: number, count: number) => T): T[];

declare function regExpString(rx: string | RegExp): string;
declare function regExpEscape(str: string): string;
declare class XRX {
    lookBehind: {
        rx: RegExp;
        expect: boolean;
    } | undefined;
    namedGroups: Record<string, number>;
    names: string[];
    rx: RegExp;
    constructor(pattern: string | RegExp, flags?: string);
    get lastIndex(): number;
    set lastIndex(index: number);
    _handleMatch(m: any): any;
    exec(str: string): any;
    execAll(str: string): any[];
    replace(str: string, replacement: string | Function): string;
}

type LogLevel = number;
declare const LogLevelAll = -1;
declare const LogLevelDebug = 0;
declare const LogLevelInfo = 1;
declare const LogLevelWarn = 2;
declare const LogLevelError = 3;
declare const LogLevelFatal = 4;
declare const LogLevelOff = 9007199254740991;
declare const LogLevelAlias: Record<string, LogLevel>;
type LogLevelAliasKey = keyof typeof LogLevelAlias;
type LogLevelAliasType = number | boolean | LogLevelAliasKey;
interface LogMessage {
    level: LogLevel;
    name: string;
    messages: any[];
    line?: number;
    file?: string;
    timestamp?: number;
}
type LogHandler = (msg: LogMessage) => void;
interface LoggerInterface {
    (...messages: any[]): void;
    /** @deprecated use .level = LogLevel.off or LogLevel.all */
    active: boolean;
    level: LogLevel;
    debug(...messages: any[]): void;
    info(...messages: any[]): void;
    warn(...messages: any[]): void;
    error(...messages: any[]): void;
    /**
     * Throws if condition is not truthy.
     *
     * **Attention!**  Due to a bug in typescript you will need to explicitly set the `LoggerInterface` type in
     * order to have the assertions having an effect on unreachable code. Example:
     *
     * ```ts
     * const log: LoggerInterface = Logger('xxx')
     * ```
     *
     * Bug https://github.com/microsoft/TypeScript/issues/50363#issuecomment-1219811447
     */
    assert(condition: unknown, ...messages: any[]): asserts condition;
    /**
     * Always throws.
     *
     * **Attention!** Due to a bug in typescript you will need to explicitly set the `LoggerInterface` type in
     * order to have the assertions having an effect on unreachable code. Example:
     *
     * ```ts
     * const log: LoggerInterface = Logger('xxx')
     * ```
     *
     * Bug https://github.com/microsoft/TypeScript/issues/50363#issuecomment-1219811447
     */
    fatal(...messages: any[]): never;
    extend(prefix: string): LoggerInterface;
    factory?: LoggerContextInterface;
    label: string;
}
interface LoggerContextInterface {
    (name?: string, level?: LogLevelAliasType): LoggerInterface;
    registerHandler(handler: LogHandler): void;
    setFilter(namespaces: string): void;
    setHandlers(handlers?: (LogHandler | undefined | null)[]): void;
    setLock(lock: boolean): void;
    /** When true emits a short log message for each Logger when being set up first time. */
    setDebug(debug: boolean): void;
    setLogLevel(level?: LogLevel): void;
    setFactory(factory: (name?: string) => LoggerInterface): void;
}
interface LogHandlerOptions {
    level?: LogLevel;
    filter?: string;
    colors?: boolean;
    levelHelper?: boolean;
    nameBrackets?: boolean;
    padding?: number;
    fill?: number;
    stack?: boolean | number;
    time?: boolean;
    pretty?: boolean;
}
declare function LoggerContext(_prefix?: string): LoggerContextInterface;

type DisposerFunction = () => any | Promise<any>;
type Disposer = DisposerFunction | {
    dispose?: Function | Promise<unknown>;
    cleanup?: Function | Promise<unknown>;
};
interface Disposable {
    dispose: UseDispose;
}
interface UseDisposeConfig {
    name?: string;
    log?: LoggerInterface;
}
declare function useDispose(config?: string | UseDisposeConfig | LoggerInterface): (() => Promise<void>) & {
    track: (obj?: Disposer) => DisposerFunction | undefined;
    add: (obj?: Disposer) => DisposerFunction | undefined;
    untrack: (disposable: Disposer) => Promise<void>;
    dispose: () => Promise<void>;
    exec: () => Promise<void>;
    getSize(): number;
    isDisposed(): boolean;
};
type UseDispose = ReturnType<typeof useDispose>;
/** @deprecated use `useDispose` instead */
declare const useDisposer: typeof useDispose;
declare function useDefer(config?: {
    mode?: 'lifo' | 'fifo';
}): ((expectSync?: boolean) => Promise<void>) & {
    add: (obj: Disposer) => void;
    exec: (expectSync?: boolean) => Promise<void>;
    getSize(): number;
};
type UseDefer = ReturnType<typeof useDefer>;
declare function useTimeout(fn: Function, timeout?: number): DisposerFunction;
declare function useInterval(fn: Function, interval: number): DisposerFunction;
declare function useEventListener(emitter: any, eventName: string, fn: (ev?: any) => void, ...args: any[]): DisposerFunction;

type Mutex = (fn: Function, elseFn?: Function) => boolean;
type AsyncMutex = (fn: Function, elseFn?: Function) => Promise<boolean>;
declare function useMutex(): Mutex;
declare function useAsyncMutex(): AsyncMutex;

type EmitterHandler = (...objs: any[]) => void;
type EmitterAllHandler<T = string> = (key: T, ...objs: any[]) => void;
declare type ListenerSignature<L> = {
    [E in keyof L]: (...args: any[]) => any;
};
declare interface DefaultListener {
    [k: string]: (...args: any[]) => any;
}
declare class Emitter<RemoteListener extends ListenerSignature<RemoteListener> = DefaultListener, LocalListener extends ListenerSignature<LocalListener> = RemoteListener> implements Disposable {
    subscribers: any;
    subscribersOnAny: any[];
    _logEmitter: LoggerInterface;
    /** Unused, but here for historical reasons */
    dispose: (() => Promise<void>) & {
        track: (obj?: Disposer | undefined) => DisposerFunction | undefined;
        add: (obj?: Disposer | undefined) => DisposerFunction | undefined;
        untrack: (disposable: Disposer) => Promise<void>;
        dispose: () => Promise<void>;
        exec: () => Promise<void>;
        getSize(): number;
        isDisposed(): boolean;
    };
    call: RemoteListener;
    emit<U extends keyof RemoteListener>(event: U, ...args: Parameters<RemoteListener[U]>): Promise<boolean>;
    onAny(fn: EmitterHandler): void;
    on<U extends keyof LocalListener>(event: U, listener: LocalListener[U]): DisposerFunction;
    onCall(handlers: Partial<LocalListener>): void;
    once<U extends keyof LocalListener>(event: U, listener: LocalListener[U]): DisposerFunction;
    off<U extends keyof LocalListener>(event: U, listener: LocalListener[U]): this;
    removeAllListeners(): this;
}
declare global {
    interface ZeedGlobalContext {
        emitter?: Emitter;
    }
    interface ZeedGlobalEmitter {
    }
}
/** Global emitter that will listen even across modules */
declare function getGlobalEmitter(): Emitter<ZeedGlobalEmitter>;
declare function lazyListener(emitter: any, listenerKey?: string): (key?: string, skipUnmatched?: boolean) => Promise<any>;

/** See http://developer.mozilla.org/en-US/docs/Web/API/MessageEvent */
interface ChannelMessageEvent {
    data: any;
    origin?: string;
    lastEventId?: string;
}
/**
 * Inspired by
 * http://developer.mozilla.org/en-US/docs/Web/API/BroadcastChannel
 * https://deno.com/deploy/docs/runtime-broadcast-channel
 * */
declare abstract class Channel extends Emitter<{
    message(event: ChannelMessageEvent): void;
    messageerror(event: ChannelMessageEvent): void;
    connect(): void;
    disconnect(): void;
    close(): void;
}> {
    id: string;
    abstract isConnected?: boolean;
    abstract postMessage(data: any): void;
    close(): void;
}
/** Very basic channel demonstrating local communication */
declare class LocalChannel extends Channel {
    isConnected: boolean;
    other?: LocalChannel;
    postMessage(data: any): void;
}
declare function createLocalChannelPair(): [LocalChannel, LocalChannel];

interface MessageAction {
    name: string;
    id: string;
    args?: Json[];
}
interface MessageResult {
    id: string;
    result?: Json;
    error?: {
        stack?: string;
        name: string;
        message: string;
    };
}
type Message = MessageAction | MessageResult;
interface MessagesOptions {
    timeout?: number;
}
interface MessagesDefaultMethods<L> {
    dispose(): void;
    connect?(channel: Channel): void;
    options(opt: MessagesOptions): L;
}
type MessagesMethods<L> = L & MessagesDefaultMethods<L>;
type MessageDefinitions = Record<any, (...args: any) => Promise<any>>;
interface MessageHub {
    dispose(): void;
    connect: (newChannel: Channel) => void;
    listen<L extends MessageDefinitions>(newHandlers: L): void;
    send<L extends MessageDefinitions>(): MessagesMethods<L>;
}
declare const createPromiseProxy: <P extends object>(fn: (name: string, args: any[], opt: any) => Promise<unknown>, opt: MessagesOptions, predefinedMethods?: any) => P;
/**
 * RPC
 *
 * Features:
 * - Waits for connection
 * - Retries after fail
 * - Timeouts
 */
declare function useMessageHub(opt?: {
    name?: string;
    channel?: Channel;
    encoder?: Encoder;
    retryAfter?: number;
    ignoreUnhandled?: boolean;
    debug?: boolean;
    logLevel?: LogLevelAliasType;
}): MessageHub;

interface PubSubConfig {
    channel: Channel;
    encoder?: Encoder;
    name?: string;
    debug?: boolean;
}
declare class PubSub<L extends ListenerSignature<L> = DefaultListener> extends Emitter<L> {
    name: string;
    channel: Channel;
    encoder: Encoder;
    log: any;
    debug: boolean;
    get shortId(): string;
    constructor(opt: PubSubConfig);
    private emitSuper;
    emit<U extends keyof L>(event: U, ...args: Parameters<L[U]>): Promise<boolean>;
    publish: <U extends keyof L>(event: U, ...args: Parameters<L[U]>) => Promise<boolean>;
    subscribe: <U extends keyof L>(event: U, listener: L[U]) => DisposerFunction;
}
declare function usePubSub<L extends ListenerSignature<L> = DefaultListener>(opt: PubSubConfig): PubSub<L>;

type ArgumentsType<T> = T extends (...args: infer A) => any ? A : never;
type ReturnType$1<T> = T extends (...args: any) => infer R ? R : never;
interface RPCOptions<Remote> {
    /** No return values expected */
    onlyEvents?: boolean;
    /** Names of remote functions that do not need response. */
    eventNames?: (keyof Remote)[];
    /** Function to post raw message */
    post: (data: any) => void;
    /** Listener to receive raw message */
    on: (fn: (data: any) => void) => void;
    /** Custom function to serialize data */
    serialize?: (data: any) => any;
    /** Custom function to deserialize data */
    deserialize?: (data: any) => any;
}
interface RPCFn<T> {
    /** Call the remote function and wait for the result. */
    (...args: ArgumentsType<T>): Promise<Awaited<ReturnType$1<T>>>;
    /** Send event without asking for response */
    asEvent(...args: ArgumentsType<T>): void;
}
type RPCReturn<RemoteFunctions> = {
    [K in keyof RemoteFunctions]: RPCFn<RemoteFunctions[K]>;
};
declare function useRPC<RemoteFunctions = {}, LocalFunctions = {}>(functions: LocalFunctions, options: RPCOptions<RemoteFunctions>): RPCReturn<RemoteFunctions>;

interface ProgressOptions {
    totalUnits?: number;
    completeUnits?: number;
    resetWhenFinished?: boolean;
    name?: string;
}
/**
 * Progress helper with these properties:
 *
 * - `totalUnits` and `completedUnits` for progress
 * - Can be cancelled
 * - Sends events on cancel, changed and dispose
 * - Supports children and propagates values.
 *   Total progress is sum of all units in the tree.
 * - On `dispose` child removes itself from parent.
 */
declare class Progress extends Emitter<{
    progressCancelled(progress: Progress): void;
    progressChanged(progress: Progress): void;
    progressDispose(progress: Progress): void;
}> {
    private _totalUnits;
    private _completedUnits;
    private _isCancelled;
    private _resetWhenFinished;
    private _children;
    name: string;
    constructor(opt?: ProgressOptions);
    /** Notify others about changes and reset on completion, if flag is set. */
    private update;
    /** Fresh start */
    reset(): void;
    /** Notify and mark as cancelled. May take some time before having an effect. */
    cancel(): Promise<void>;
    /** Add child progress, which count into its parents units. On `dispose` it will auto remove itself from parent. */
    addChild(child: Progress): void;
    /** Create child progress.  */
    createChildProgress(opt?: ProgressOptions): Progress;
    removeChild(child: Progress): void;
    /** Total units including children */
    getTotalUnits(): number;
    /** Completed units including children */
    getCompletedUnits(): number;
    /** Cannot calculate progress, because totalUnits are missing. Special representation in UI. */
    isIndeterminate(): boolean;
    isCancelled(): boolean;
    /** Either disposed or all units completed. */
    isFinished(): boolean;
    /** Value from 0 to 1, where 1 is 100% completeness. */
    getFraction(): number;
    getChildrenCount(): number;
    /** Change total units. */
    setTotalUnits(units: number, completedUnits?: number): void;
    /** Relatively change total units. */
    incTotalUnits(step?: number): void;
    /** Set fixed number of completed units. */
    setCompletetedUnits(units: number): void;
    /** Set to 100% without disposing. */
    setCompleted(): void;
    /** Dynamically change completed units. */
    incCompletedUnits(step?: number): void;
    /** Progress tree to string for debuggin purposes. Consider using `name` attribute of Progress. */
    toString(indent?: number): string;
}

type PoolTaskIdConflictResolution = 'replace' | 'memoize' | 'prioritize' | 'error';
interface PoolConfig {
    name?: string;
    maxParallel?: number;
    idConflictResolution?: PoolTaskIdConflictResolution;
}
type PoolTaskFn<T = any> = (taskInfo: PoolTask<T>) => Promise<T>;
type PoolTaskState = 'waiting' | 'running' | 'finished';
/** Task */
interface PoolTask<T> {
    readonly id: string;
    readonly task: PoolTaskFn<T>;
    readonly done: Function;
    readonly setMax: (max: number) => void;
    readonly setResolved: (resolved: number) => void;
    readonly incResolved: (inc?: number) => void;
    state: PoolTaskState;
    priority: number;
    /** Same groups are executed only one at a time */
    group?: string;
    progress: Progress;
    max: number;
    resolved: number;
    result?: T;
    payload?: unknown;
}
interface PoolTaskEvents {
    didUpdate(max: number, resolved: number, presentMax: number, presentResolved: number): void;
    didStart(id: string): void;
    didCancel(id: string): void;
    didFinish(): void;
    didResolve(id: string, value: any): void;
    didReject(id: string, error: any): void;
}
declare function usePool<T = any>(config?: PoolConfig): {
    events: Emitter<PoolTaskEvents, PoolTaskEvents>;
    cancel: (id: string) => void;
    cancelAll: () => void;
    hasById: (id: string) => boolean;
    progress: Progress;
    enqueue: <P>(task: PoolTaskFn<T>, config?: {
        id?: string | undefined;
        max?: number | undefined;
        resolved?: number | undefined;
        group?: string | undefined;
        idConflictResolution?: PoolTaskIdConflictResolution | undefined;
        payload?: P | undefined;
    }) => {
        id: string;
        promise: Promise<any>;
        dispose: () => void;
        cancel: () => void;
    };
    dispose: (() => Promise<void>) & {
        track: (obj?: Disposer | undefined) => DisposerFunction | undefined;
        add: (obj?: Disposer | undefined) => DisposerFunction | undefined;
        untrack: (disposable: Disposer) => Promise<void>;
        dispose: () => Promise<void>;
        exec: () => Promise<void>;
        getSize(): number;
        isDisposed(): boolean;
    };
    waitFinishAll: () => Promise<unknown>;
};

/**
 * Promise to be used with `await`. Example:
 *
 * ```
 * const [promise, resolve, reject] = createPromise()
 * setTimeout(() => {
 *   resolve(5)
 * }, 50)
 * const result = await promise
 * ```
 */
declare function createPromise<T>(): [Promise<T>, any, any];
/** Sleep for `milliSeconds`. Example 1s: `await sleep(1000)` */
declare function sleep(milliSeconds: number): Promise<void>;
/** Same as `await sleep(0)`, just let the event loop execute. */
declare function immediate(): Promise<void>;
declare function timeout<T>(promise: Promise<T>, milliSeconds: number, timeoutValue?: string): Promise<T | typeof timeoutValue>;
declare function isTimeout(value: any): boolean;
declare function tryTimeout<T>(promise: Promise<T>, milliSeconds: number): Promise<T | undefined>;
/** Wait for `event` on `obj` to emit. Resolve with result or reject on `timeout` */
declare function waitOn(obj: any, event: string, timeoutMS?: number): Promise<any>;
declare function isPromise<T>(value: Promise<T> | T): value is Promise<T>;
/** This is exactly what Prose.resolve(x) is supposed to be: return a Promise no matter what type x is */
declare function promisify<T>(value: Promise<T> | T): Promise<T>;
/**
 *  Like ReturnType but for async functions.
 *  From https://www.jpwilliams.dev/how-to-unpack-the-return-type-of-a-promise-in-typescript
 */
type AsyncReturnType<T extends (...args: any) => any> = T extends (...args: any) => Promise<infer U> ? U : T extends (...args: any) => infer U ? U : any;

type TaskFn<T = any> = () => Promise<T>;
interface TaskEvents {
    didUpdate(max: number, resolved: number): void;
    didStart(max: number): void;
    didCancel(): void;
    didFinish(): void;
}
/** Guarentee serial execution of tasks. Able to wait, pause, resume and cancel all. */
declare class SerialQueue extends Emitter<TaskEvents> {
    private queue;
    private waitToFinish;
    private currentTask?;
    private log;
    private countMax;
    private countResolved;
    private paused;
    name: string;
    constructor(opt?: {
        name?: string;
        logLevel?: LogLevel;
    });
    private performNext;
    /** Enqueue task to be executed when all other tasks are done. Except `immediate = true`. */
    enqueue<T>(task: TaskFn<T>, opt?: {
        immediate?: boolean;
        name?: string;
    }): Promise<T>;
    /** If a task is already performing, execute immediately. Otherwise enqueue as usual. */
    enqueueReentrant<T>(task: TaskFn<T>, opt?: {
        name?: string;
    }): Promise<T>;
    /** Remove all tasks from queue that are not yet executing. */
    cancelAll(_unblock?: boolean): Promise<void>;
    /** Pause execution after current task is finished. */
    pause(): Promise<void>;
    /** Resume paused queue. */
    resume(): void;
    /** Wait for all tasks to finish */
    wait(): Promise<unknown>;
    get isPaused(): boolean;
    get hasTasks(): boolean;
}

/**
 * A special throttle implementation that tries to distribute execution
 * in an optimal way.
 *
 * **Functionality:** For UI usage the function is executed on first occasion (`leading`).
 * If more calls follow it will again be executed at end (`trailing`).
 * If the next call is inside the timeframe, it is delayed until `trailing`.
 * This avoids timewise too close calls.
 * It is possible to `cancel` the timeout and to `flush` a call, e.g. if
 * leaving UI situation where a final call is required to write data or similar.
 */
declare function throttle<F extends (...args: any[]) => any>(callback: F, opt?: {
    delay?: number;
    trailing?: boolean;
    leading?: boolean;
}): F & {
    /** Stop all timers, do not exec nothing */
    cancel: () => void;
    /** Stop all timers and execute right now. */
    immediate: (...args: Parameters<F>) => Promise<void>;
    /** Stop all timers and execute trailing call, if exists. */
    stop: () => void;
    dispose: () => void;
};
/**
 * Debounce fits best for filtering a large peak of events.
 * For UI event filtering throttle is probably a better choice.
 *
 * **Functionality:**  It only fires after triggers pause for `delay` ms.
 */
declare function debounce<F extends (...args: any[]) => any | Promise<any>>(callback: F, opt?: {
    delay?: number;
}): F & {
    cancel: () => void;
    immediate: (...args: Parameters<F>) => Promise<void>;
    dispose: () => void;
};

declare global {
    interface ZeedGlobalContext {
    }
}
declare function getGlobalContext(): ZeedGlobalContext;

declare function isLocalHost(hostname?: string): boolean;

declare global {
    interface ZeedGlobalContext {
        logger?: any;
    }
}
declare function getGlobalLogger(setup?: (context: LoggerContextInterface) => void): LoggerContextInterface;
/** Can be used for lazy usage e.g. inside of Zeed  */
declare function getGlobalLoggerIfExists(): any;
declare function DefaultLogger(name?: string, level?: LogLevelAliasType): LoggerInterface;

/**
 * Very basic logger. Please take a look at the browser and node
 * optimized loggers. This one is just the absolute fallback option.
 *
 * @param level Log level
 * @returns Logger
 */
declare function LoggerConsoleHandler(opt?: LogHandlerOptions): LogHandler;

interface NamespaceFilter {
    (name: string): boolean;
    accept: RegExp[];
    reject: RegExp[];
    filter: string;
}
declare function getNamespaceFilterString(defaultNamespaceFilter: any): string;
/**
 * Filter as described here https://github.com/visionmedia/debug#wildcards
 *
 * @param filter Namespace filter
 * @returns Function to check if filter applies
 */
declare function useNamespaceFilter(filter?: string): NamespaceFilter;
declare function parseLogLevel(filter: LogLevelAliasType): LogLevel;
declare function useLevelFilter(filter?: string | number | boolean | LogLevelAliasType): (level: number) => boolean;

/**
 * Collect messages in a list.
 *
 * @param level Log level
 * @returns Logger
 */
declare function LoggerMemoryHandler(opt: LogHandlerOptions & {
    messages: LogMessage[];
}): LogHandler;

interface fetchOptionType {
    /** Returns the cache mode associated with request, which is a string indicating how the request will interact with the browser's cache when fetching. */
    cache?: RequestCache;
    /** Returns the credentials mode associated with request, which is a string indicating whether credentials will be sent with the request always, never, or only when sent to a same-origin URL. */
    credentials?: RequestCredentials;
    /** Returns the kind of resource requested by request, e.g., "document" or "script". */
    destination?: RequestDestination;
    /** Returns a Headers object consisting of the headers associated with request. Note that headers added in the network layer by the user agent will not be accounted for in this object, e.g., the "Host" header. */
    headers?: Record<string, string>;
    /** Returns request's subresource integrity metadata, which is a cryptographic hash of the resource being fetched. Its value consists of multiple hashes separated by whitespace. [SRI] */
    integrity?: string;
    /** Returns a boolean indicating whether or not request can outlive the global in which it was created. */
    keepalive?: boolean;
    /** Returns request's HTTP method, which is "GET" by default. */
    method?: string;
    /** Returns the mode associated with request, which is a string indicating whether the request will use CORS, or will be restricted to same-origin URLs. */
    mode?: RequestMode;
    /** Returns the redirect mode associated with request, which is a string indicating how redirects for the request will be handled during fetching. A request will follow redirects by default. */
    redirect?: RequestRedirect;
    /** Returns the referrer of request. Its value can be a same-origin URL if explicitly set in init, the empty string to indicate no referrer, and "about:client" when defaulting to the global's default. This is used during fetching to determine the value of the `Referer` header of the request being made. */
    referrer?: string;
    /** Returns the referrer policy associated with request. This is used during fetching to compute the value of the request's referrer. */
    referrerPolicy?: ReferrerPolicy;
    /** Returns the signal associated with request, which is an AbortSignal object indicating whether or not request has been aborted, and its abort event handler. */
    signal?: AbortSignal;
    /** Returns the URL of request as a string. */
    url?: string;
    body?: any;
}
type fetchOptionsType = fetchOptionType | fetchOptionsType[];
type httpMethod = 'GET' | 'POST' | 'PUT' | 'DELETE' | 'HEAD' | 'CONNECT' | 'OPTIONS' | 'TRACE' | 'PATCH';
declare function parseBasicAuth(url: string): {
    url: string;
    username: string;
    password: string;
} | undefined;
/** Simplified `fetch` that returns `undefined` on non 200 status */
declare function fetchBasic(url: string | URL, fetchOptions?: fetchOptionsType, fetchFn?: (input: RequestInfo, init?: RequestInit) => Promise<Response>): Promise<Response | undefined>;
/** Fetch for JSON  */
declare function fetchJson<T = Json>(url: string | URL, fetchOptions?: fetchOptionsType, fetchFn?: (input: RequestInfo, init?: RequestInit) => Promise<Response>): Promise<T | undefined>;
/** Fetch for text */
declare function fetchText(url: string | URL, fetchOptions?: fetchOptionsType, fetchFn?: (input: RequestInfo, init?: RequestInit) => Promise<Response>): Promise<string | undefined>;
/** Options for fetchBasic to send data as application/x-www-form-urlencoded */
declare function fetchOptionsFormURLEncoded(data: object, method?: httpMethod): fetchOptionType;
/** Options to send data as JSON  */
declare function fetchOptionsJson(data: object, method?: httpMethod): fetchOptionType;
/** Options to pass basic auth */
declare function fetchOptionsBasicAuth(username: string, password: string): fetchOptionType;

declare function getWindow(): any | undefined;
declare function getNavigator(): any | undefined;
declare function getGlobal(): any;
declare function detect(info?: {
    ios: boolean;
    macos: boolean;
    windows: boolean;
    beaker: boolean;
    electron: boolean;
    wkwebview: boolean;
    pwa: boolean;
    pwaInstalled: boolean;
    browser: boolean;
    node: boolean;
    worker: boolean;
    test: boolean;
    jest: boolean;
    macosNative: boolean;
    iosNative: boolean;
    appleNative: boolean;
    touch: boolean;
}): {
    ios: boolean;
    macos: boolean;
    windows: boolean;
    beaker: boolean;
    electron: boolean;
    wkwebview: boolean;
    pwa: boolean;
    pwaInstalled: boolean;
    browser: boolean;
    node: boolean;
    worker: boolean;
    test: boolean;
    jest: boolean;
    macosNative: boolean;
    iosNative: boolean;
    appleNative: boolean;
    touch: boolean;
};
declare function isBrowser(): boolean;
/**
 * Before closing the tab/window or quitting the node process
 * allow to do something important first
 */
declare function useExitHandler(handler: () => void): void;

interface MemStorageOptions {
    objectFromString?: (data: string) => any;
    objectToString?: (data: any) => string;
}
declare class MemStorage<T = Json> implements ObjectStorage<T> {
    private store;
    constructor(_opt?: MemStorageOptions);
    setItem(key: string, value: T): void;
    getItem(key: string): T | undefined;
    removeItem(key: string): void;
    clear(): void;
    allKeys(): string[];
}

/**
 * @returns Timestamp in miliseconds
 */
declare const getTimestamp: () => number;
declare function formatMilliseconds(ms: number): string;
declare function parseDate(...dateCandidates: (string | Date)[]): Date | undefined;

declare const noop: () => void;

declare function uuidBytes(): Uint8Array;
declare function uuidB62(bytes?: Uint8Array): string;
declare function uuidEncodeB62(bytes: Uint8Array): string;
declare function uuidDecodeB62(uuid: string): Uint8Array;
declare function uuidB32(bytes?: Uint8Array): string;
declare function uuidEncodeB32(bytes: Uint8Array): string;
declare function uuidDecodeB32(uuid: string): Uint8Array;
declare const uuidv4: () => string;
declare function uuidEncodeV4(bytes: Uint8Array): string;
declare function uuidDecodeV4(uuid: string): Uint8Array;
declare function suidBytes(): Uint8Array;
declare function suid(): string;
declare function suidDate(id: string): Date;
declare function suidBytesDate(id: Uint8Array): Date;
declare function uuid32bit(): number;
declare const mapModes: {
    base62: {
        uuid: typeof uuidB62;
        uuidDecode: typeof uuidDecodeB62;
        uuidEncode: typeof uuidEncodeB62;
    };
    base32: {
        uuid: typeof uuidB32;
        uuidDecode: typeof uuidDecodeB32;
        uuidEncode: typeof uuidEncodeB32;
    };
    uuidv4: {
        uuid: () => string;
        uuidDecode: typeof uuidDecodeV4;
        uuidEncode: typeof uuidEncodeV4;
    };
};
declare function setUuidDefaultEncoding(mode: keyof typeof mapModes, sorted?: boolean): void;
declare function uuid(): string;
declare function uuidDecode(uuid: string): Uint8Array;
declare function uuidEncode(bytes: Uint8Array): string;
declare function uuidIsValid(uuid: string): boolean;
declare function uname(name?: string): string;
declare function qid(): string;

export { _useBase as $, arrayUnion as A, BinaryEncoder as B, CRYPTO_DEFAULT_HASH_ALG as C, arrayFlatten as D, arrayIntersection as E, arraySymmetricDifference as F, arrayRemoveElement as G, arraySetElement as H, arrayFilterInPlace as I, Json as J, arrayToggleInPlace as K, LogHandlerOptions as L, arrayEmptyInPlace as M, arraySorted as N, ObjectStorage as O, arraySortedNumbers as P, arrayIsEqual as Q, arrayShuffleInPlace as R, arrayShuffle as S, arrayShuffleForce as T, arrayRandomElement as U, arrayMax as V, arrayMin as W, arraySum as X, arrayAvg as Y, arrayBatches as Z, createArray as _, LogHandler as a, dayFromString as a$, useBase as a0, encodeBase32 as a1, decodeBase32 as a2, encodeBase16 as a3, decodeBase16 as a4, encodeBase58 as a5, decodeBase58 as a6, encodeBase62 as a7, decodeBase62 as a8, estimateSizeForBase as a9, valueToFloat as aA, valueToString as aB, toFloat as aC, toInt as aD, toString as aE, toBool as aF, RenderMessagesOptions as aG, formatMessages as aH, renderMessages as aI, fixBrokenUth8String as aJ, DAY_MS as aK, DayValue as aL, DayInput as aM, dayYear as aN, dayMonth as aO, dayDay as aP, dayToParts as aQ, dayToDate as aR, dayFromToday as aS, dayFromAny as aT, dayToDateGMT as aU, dayFromDate as aV, dayFromDateGMT as aW, dayToTimestamp as aX, dayFromTimestamp as aY, dayToString as aZ, dayFromParts as a_, BinInput as aa, _encodeUtf8Polyfill as ab, _decodeUtf8Polyfill as ac, stringToUInt8Array as ad, Uint8ArrayToString as ae, toUint8Array as af, joinToUint8Array as ag, toHex as ah, fromHex as ai, toBase64 as aj, toBase64Url as ak, fromBase64 as al, fromBase64String as am, equalBinary as an, jsonToUint8Array as ao, Uint8ArrayToJson as ap, Uint8ArrayToHexDump as aq, toCamelCase as ar, toCapitalize as as, toCapitalizeWords as at, fromCamelCase as au, stringToBoolean as av, stringToInteger as aw, stringToFloat as ax, valueToBoolean as ay, valueToInteger as az, LogLevelAliasType as b, objectMergeDisposable as b$, dayMonthStart as b0, dayYearStart as b1, dayOffset as b2, dayDiff as b3, dayRange as b4, dayIterator as b5, DayInputLegacy as b6, Day as b7, forEachDay as b8, today as b9, isBoolean as bA, isNullOrUndefined as bB, isNull as bC, isUint8Array as bD, isNotNull as bE, isValue as bF, jsonStringifySorted as bG, jsonStringifySafe as bH, jsonStringify as bI, FilterFunction as bJ, MapperFunction as bK, listQuery as bL, listGroupBy as bM, listDistinctUnion as bN, listOfKey as bO, getSecureRandom as bP, getSecureRandomIfPossible as bQ, randomBoolean as bR, randomInt as bS, randomFloat as bT, between as bU, sum as bV, avg as bW, isPrimeRX as bX, isPrime as bY, seededRandom as bZ, objectMap as b_, day as ba, dateStringToDays as bb, DecimalValue as bc, DecimalInput as bd, decimal as be, decimalFromCents as bf, decimalToCents as bg, decimalCentsPart as bh, deepEqual as bi, deepStripUndefinedInPlace as bj, deepMerge as bk, DifferenceType as bl, Difference as bm, diffObjects as bn, escapeHTML as bo, unescapeHTML as bp, Primitive as bq, isObject as br, isPrimitive as bs, isArray as bt, isRecord as bu, isRecordPlain as bv, isString as bw, isNumber as bx, isInteger as by, isSafeInteger as bz, LoggerInterface as c, Progress as c$, objectIsEmpty as c0, parseOrderby as c1, composeOrderby as c2, cmp as c3, sortedOrderby as c4, toValidFilename as c5, escapeRegExp as c6, RoundingMode as c7, isHalf as c8, isEven as c9, memoizeAsync as cA, forTimes as cB, regExpString as cC, regExpEscape as cD, XRX as cE, DisposerFunction as cF, Disposer as cG, Disposable as cH, useDispose as cI, UseDispose as cJ, useDisposer as cK, useDefer as cL, UseDefer as cM, useTimeout as cN, useInterval as cO, useEventListener as cP, Mutex as cQ, AsyncMutex as cR, useMutex as cS, useAsyncMutex as cT, PoolTaskIdConflictResolution as cU, PoolConfig as cV, PoolTaskFn as cW, PoolTaskState as cX, PoolTask as cY, PoolTaskEvents as cZ, usePool as c_, roundUp as ca, roundDown as cb, roundHalfUp as cc, roundHalfOdd as cd, roundHalfAwayFromZero as ce, roundHalfDown as cf, roundHalfEven as cg, roundHalfTowardsZero as ch, SortableItem as ci, startSortWeight as cj, endSortWeight as ck, moveSortWeight as cl, sortedItems as cm, useSorted as cn, linkifyPlainText as co, toHumanReadableUrl as cp, encodeQuery as cq, parseQuery as cr, ensureKey as cs, ensureKeyAsync as ct, size as cu, last as cv, empty as cw, cloneObject as cx, cloneJsonObject as cy, memoize as cz, assert as d, createPromiseProxy as d$, createPromise as d0, sleep as d1, immediate as d2, timeout as d3, isTimeout as d4, tryTimeout as d5, waitOn as d6, isPromise as d7, promisify as d8, AsyncReturnType as d9, useNamespaceFilter as dA, parseLogLevel as dB, useLevelFilter as dC, LoggerMemoryHandler as dD, ChannelMessageEvent as dE, Channel as dF, LocalChannel as dG, createLocalChannelPair as dH, EmitterHandler as dI, EmitterAllHandler as dJ, ListenerSignature as dK, DefaultListener as dL, Emitter as dM, getGlobalEmitter as dN, lazyListener as dO, Encoder as dP, NoopEncoder as dQ, JsonEncoder as dR, CryptoEncoder as dS, MessageAction as dT, MessageResult as dU, Message as dV, MessagesOptions as dW, MessagesDefaultMethods as dX, MessagesMethods as dY, MessageDefinitions as dZ, MessageHub as d_, TaskFn as da, TaskEvents as db, SerialQueue as dc, throttle as dd, debounce as de, getGlobalContext as df, isLocalHost as dg, getGlobalLogger as dh, getGlobalLoggerIfExists as di, DefaultLogger as dj, LogLevel as dk, LogLevelAll as dl, LogLevelDebug as dm, LogLevelInfo as dn, LogLevelWarn as dp, LogLevelError as dq, LogLevelFatal as dr, LogLevelOff as ds, LogLevelAlias as dt, LogLevelAliasKey as du, LogMessage as dv, LoggerContextInterface as dw, LoggerContext as dx, LoggerConsoleHandler as dy, getNamespaceFilterString as dz, assertCondition as e, useMessageHub as e0, PubSub as e1, usePubSub as e2, ArgumentsType as e3, ReturnType$1 as e4, RPCOptions as e5, RPCFn as e6, RPCReturn as e7, useRPC as e8, httpMethod as e9, uuidDecodeB32 as eA, uuidv4 as eB, uuidEncodeV4 as eC, uuidDecodeV4 as eD, suidBytes as eE, suid as eF, suidDate as eG, suidBytesDate as eH, uuid32bit as eI, setUuidDefaultEncoding as eJ, uuid as eK, uuidDecode as eL, uuidEncode as eM, uuidIsValid as eN, uname as eO, qid as eP, parseBasicAuth as ea, fetchBasic as eb, fetchJson as ec, fetchText as ed, fetchOptionsFormURLEncoded as ee, fetchOptionsJson as ef, fetchOptionsBasicAuth as eg, getWindow as eh, getNavigator as ei, getGlobal as ej, detect as ek, isBrowser as el, useExitHandler as em, MemStorageOptions as en, MemStorage as eo, getTimestamp as ep, formatMilliseconds as eq, parseDate as er, NestedArray as es, noop as et, uuidBytes as eu, uuidB62 as ev, uuidEncodeB62 as ew, uuidDecodeB62 as ex, uuidB32 as ey, uuidEncodeB32 as ez, fatal as f, encodeJson as g, decodeJson as h, createBinaryStreamEncoder as i, createBinaryStreamDecoder as j, bitfield as k, blobToArrayBuffer as l, blobToUint8Array as m, CRYPTO_DEFAULT_ALG as n, CRYPTO_DEFAULT_DERIVE_ALG as o, CRYPTO_DEFAULT_DERIVE_ITERATIONS as p, digest as q, randomUint8Array as r, deriveKeyPbkdf2 as s, encrypt as t, decrypt as u, csvStringify as v, csvParse as w, csvParseToObjects as x, arrayUnique as y, arrayMinus as z };
